@@ -23,6 +23,7 @@ void add_cylinder_description(cylinder_type_t *type)
 			return;
 	}
 	if (i < 100) {
+		// FIXME: leaked on exit
 		tank_info[i].name = strdup(desc);
 		tank_info[i].ml = type->size.mliter;
 		tank_info[i].bar = type->workingpressure.mbar / 1000;
@@ -43,6 +44,7 @@ void add_weightsystem_description(weightsystem_t *weightsystem)
 		}
 	}
 	if (i < 100) {
+		// FIXME: leaked on exit
 		ws_info[i].name = strdup(desc);
 		ws_info[i].grams = weightsystem->weight.grams;
 	}
@@ -77,8 +79,10 @@ void get_gas_string(const struct gasmix *gasmix, char *text, int len)
 {
 	if (gasmix_is_air(gasmix))
 		snprintf(text, len, "%s", translate("gettextFromC", "air"));
-	else if (get_he(gasmix) == 0)
+	else if (get_he(gasmix) == 0 && get_o2(gasmix) < 1000)
 		snprintf(text, len, translate("gettextFromC", "EAN%d"), (get_o2(gasmix) + 5) / 10);
+	else if (get_he(gasmix) == 0 && get_o2(gasmix) == 1000)
+		snprintf(text, len, "%s", translate("gettextFromC", "oxygen"));
 	else
 		snprintf(text, len, "(%d/%d)", (get_o2(gasmix) + 5) / 10, (get_he(gasmix) + 5) / 10);
 }
