@@ -224,11 +224,15 @@ static void update_cylinder_pressure(struct dive *d, int old_depth, int new_dept
 	volume_t gas_used;
 	pressure_t delta_p;
 	depth_t mean_depth;
+	int factor = 1000;
+
+	if (d->dc.divemode == PSCR)
+		factor = prefs.pscr_ratio;
 
 	if (!cyl)
 		return;
 	mean_depth.mm = (old_depth + new_depth) / 2;
-	gas_used.mliter = depth_to_atm(mean_depth.mm, d) * sac / 60 * duration;
+	gas_used.mliter = depth_to_atm(mean_depth.mm, d) * sac / 60 * duration * factor / 1000;
 	cyl->gas_used.mliter += gas_used.mliter;
 	if (in_deco)
 		cyl->deco_gas_used.mliter += gas_used.mliter;
@@ -251,7 +255,7 @@ static void create_dive_from_plan(struct diveplan *diveplan, bool track_gas)
 	int oldpo2 = 0;
 	int lasttime = 0;
 	int lastdepth = 0;
-	enum dive_comp_type type = OC;
+	enum dive_comp_type type = displayed_dive.dc.divemode;
 
 	if (!diveplan || !diveplan->dp)
 		return;
@@ -278,6 +282,7 @@ static void create_dive_from_plan(struct diveplan *diveplan, bool track_gas)
 	oldgasmix = cyl->gasmix;
 	sample = prepare_sample(dc);
 	sample->setpoint.mbar = dp->setpoint;
+	sample->sac.mliter = prefs.bottomsac;
 	oldpo2 = dp->setpoint;
 	if (track_gas && cyl->type.workingpressure.mbar)
 		sample->cylinderpressure.mbar = cyl->end.mbar;
@@ -323,6 +328,7 @@ static void create_dive_from_plan(struct diveplan *diveplan, bool track_gas)
 			sample->time.seconds = lasttime + 1;
 			sample->depth.mm = lastdepth;
 			sample->manually_entered = dp->entered;
+			sample->sac.mliter = dp->entered ? prefs.bottomsac : prefs.decosac;
 			if (track_gas && cyl->type.workingpressure.mbar)
 				sample->cylinderpressure.mbar = cyl->sample_end.mbar;
 			finish_sample(dc);
@@ -337,6 +343,7 @@ static void create_dive_from_plan(struct diveplan *diveplan, bool track_gas)
 		sample->time.seconds = lasttime = time;
 		sample->depth.mm = lastdepth = depth;
 		sample->manually_entered = dp->entered;
+		sample->sac.mliter = dp->entered ? prefs.bottomsac : prefs.decosac;
 		if (track_gas && !sample[-1].setpoint.mbar) {    /* Don't track gas usage for CCR legs of dive */
 			update_cylinder_pressure(&displayed_dive, sample[-1].depth.mm, depth, time - sample[-1].time.seconds,
 					dp->entered ? diveplan->bottomsac : diveplan->decosac, cyl, !dp->entered);
@@ -346,7 +353,7 @@ static void create_dive_from_plan(struct diveplan *diveplan, bool track_gas)
 		finish_sample(dc);
 		dp = dp->next;
 	}
-	dc->dctype = type;
+	dc->divemode = type;
 #if DEBUG_PLAN & 32
 	save_dive(stdout, &displayed_dive);
 #endif
@@ -606,9 +613,9 @@ static void add_plan_to_notes(struct diveplan *diveplan, struct dive *dive, bool
 							 gasname(&gasmix));
 
 					len += snprintf(buffer + len, sizeof(buffer) - len, "%s<br>", temp);
-					newdepth = dp->depth;
-					lasttime = dp->time;
 				}
+				newdepth = dp->depth;
+				lasttime = dp->time;
 			} else {
 				if (dp->depth != nextdp->depth || gasmix_distance(&gasmix, &newgasmix) != 0 || dp->setpoint != nextdp->setpoint) {
 					if (dp->setpoint)
@@ -699,7 +706,7 @@ static void add_plan_to_notes(struct diveplan *diveplan, struct dive *dive, bool
 	snprintf(temp, sizeof(temp), "%s", translate("gettextFromC", "OTU"));
 	len += snprintf(buffer + len, sizeof(buffer) - len, "<br>%s: %i</div>", temp, dive->otu);
 
-	if (dive->dc.dctype == CCR)
+	if (dive->dc.divemode == CCR)
 		snprintf(temp, sizeof(temp), "%s", translate("gettextFromC", "Gas consumption (CCR legs excluded):"));
 	else
 		snprintf(temp, sizeof(temp), "%s", translate("gettextFromC", "Gas consumption:"));
@@ -737,23 +744,37 @@ static void add_plan_to_notes(struct diveplan *diveplan, struct dive *dive, bool
 		len += snprintf(buffer + len, sizeof(buffer) - len, "%s%s<br>", temp, warning);
 	}
 	dp = diveplan->dp;
-	while (dp) {
-		if (dp->time != 0) {
-			int pO2 = depth_to_atm(dp->depth, dive) * get_o2(&dp->gasmix);
+	if (dive->dc.divemode != CCR) {
+		while (dp) {
+			if (dp->time != 0) {
+				struct gas_pressures pressures;
+				fill_pressures(&pressures, depth_to_atm(dp->depth, dive), &dp->gasmix, 0.0, dive->dc.divemode);
 
-			if (pO2 > (dp->entered ? prefs.bottompo2 : prefs.decopo2)) {
-				const char *depth_unit;
-				int decimals;
-				double depth_value = get_depth_units(dp->depth, &decimals, &depth_unit);
-				len = strlen(buffer);
-				snprintf(temp, sizeof(temp),
-					 translate("gettextFromC", "high pO₂ value %.2f at %d:%02u with gas %s at depth %.*f %s"),
-					 pO2 / 1000.0, FRACTION(dp->time, 60), gasname(&dp->gasmix), decimals, depth_value, depth_unit);
-				len += snprintf(buffer + len, sizeof(buffer) - len, "<span style='color: red;'>%s </span> %s<br>",
-						translate("gettextFromC", "Warning:"), temp);
+				if (pressures.o2 > (dp->entered ? prefs.bottompo2 : prefs.decopo2) / 1000.0) {
+					const char *depth_unit;
+					int decimals;
+					double depth_value = get_depth_units(dp->depth, &decimals, &depth_unit);
+					len = strlen(buffer);
+					snprintf(temp, sizeof(temp),
+						 translate("gettextFromC", "high pO₂ value %.2f at %d:%02u with gas %s at depth %.*f %s"),
+						 pressures.o2, FRACTION(dp->time, 60), gasname(&dp->gasmix), decimals, depth_value, depth_unit);
+					len += snprintf(buffer + len, sizeof(buffer) - len, "<span style='color: red;'>%s </span> %s<br>",
+							translate("gettextFromC", "Warning:"), temp);
+				} else if (pressures.o2 < 0.16) {
+					const char *depth_unit;
+					int decimals;
+					double depth_value = get_depth_units(dp->depth, &decimals, &depth_unit);
+					len = strlen(buffer);
+					snprintf(temp, sizeof(temp),
+						 translate("gettextFromC", "low pO₂ value %.2f at %d:%02u with gas %s at depth %.*f %s"),
+						 pressures.o2, FRACTION(dp->time, 60), gasname(&dp->gasmix), decimals, depth_value, depth_unit);
+					len += snprintf(buffer + len, sizeof(buffer) - len, "<span style='color: red;'>%s </span> %s<br>",
+							translate("gettextFromC", "Warning:"), temp);
+
+				}
 			}
+			dp = dp->next;
 		}
-		dp = dp->next;
 	}
 	snprintf(buffer + len, sizeof(buffer) - len, "</div>");
 	dive->notes = strdup(buffer);
