@@ -43,6 +43,7 @@ struct plot_data {
 	struct gas_pressures pressures;
 	pressure_t o2pressure;  // for rebreathers, this is consensus measured po2, or setpoint otherwise. 0 for OC.
 	pressure_t o2sensor[3]; //for rebreathers with up to 3 PO2 sensors
+	pressure_t o2setpoint;
 	double mod, ead, end, eadd;
 	velocity_t velocity;
 	int speed;
@@ -67,7 +68,7 @@ struct ev_select {
 	bool plot_ev;
 };
 
-struct plot_info calculate_max_limits_new(struct dive *dive, struct divecomputer *dc);
+struct plot_info calculate_max_limits_new(struct dive *dive, struct divecomputer *given_dc);
 void compare_samples(struct plot_data *e1, struct plot_data *e2, char *buf, int bufsize, int sum);
 struct plot_data *populate_plot_entries(struct dive *dive, struct divecomputer *dc, struct plot_info *pi);
 struct plot_info *analyze_plot_info(struct plot_info *pi);
