@@ -1,14 +1,13 @@
 #include "configuredivecomputerdialog.h"
 
-#include "../divecomputer.h"
-#include "../libdivecomputer.h"
-#include "../helpers.h"
-#include "../display.h"
-#include "../divelist.h"
-#include "configuredivecomputer.h"
+#include "helpers.h"
+#include "mainwindow.h"
 #include <QFileDialog>
 #include <QMessageBox>
 #include <QSettings>
+#include <QNetworkReply>
+#include <QProgressDialog>
+
 struct product {
 	const char *product;
 	dc_descriptor_t *descriptor;
@@ -28,10 +27,14 @@ struct mydescriptor {
 	unsigned int model;
 };
 
-GasSpinBoxItemDelegate::GasSpinBoxItemDelegate(QObject *parent, column_type type) : QStyledItemDelegate(parent), type(type) { }
-GasSpinBoxItemDelegate::~GasSpinBoxItemDelegate() { }
+GasSpinBoxItemDelegate::GasSpinBoxItemDelegate(QObject *parent, column_type type) : QStyledItemDelegate(parent), type(type)
+{
+}
+GasSpinBoxItemDelegate::~GasSpinBoxItemDelegate()
+{
+}
 
-QWidget* GasSpinBoxItemDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
+QWidget *GasSpinBoxItemDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
 	// Create the spinbox and give it it's settings
 	QSpinBox *sb = new QSpinBox(parent);
@@ -49,7 +52,7 @@ QWidget* GasSpinBoxItemDelegate::createEditor(QWidget *parent, const QStyleOptio
 
 void GasSpinBoxItemDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
 {
-	if(QSpinBox *sb = qobject_cast<QSpinBox *>(editor))
+	if (QSpinBox *sb = qobject_cast<QSpinBox *>(editor))
 		sb->setValue(index.data(Qt::EditRole).toInt());
 	else
 		QStyledItemDelegate::setEditorData(editor, index);
@@ -58,16 +61,20 @@ void GasSpinBoxItemDelegate::setEditorData(QWidget *editor, const QModelIndex &i
 
 void GasSpinBoxItemDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
 {
-	if(QSpinBox *sb = qobject_cast<QSpinBox *>(editor))
+	if (QSpinBox *sb = qobject_cast<QSpinBox *>(editor))
 		model->setData(index, sb->value(), Qt::EditRole);
 	else
 		QStyledItemDelegate::setModelData(editor, model, index);
 }
 
-GasTypeComboBoxItemDelegate::GasTypeComboBoxItemDelegate(QObject *parent, computer_type type) : QStyledItemDelegate(parent), type(type) { }
-GasTypeComboBoxItemDelegate::~GasTypeComboBoxItemDelegate() { }
+GasTypeComboBoxItemDelegate::GasTypeComboBoxItemDelegate(QObject *parent, computer_type type) : QStyledItemDelegate(parent), type(type)
+{
+}
+GasTypeComboBoxItemDelegate::~GasTypeComboBoxItemDelegate()
+{
+}
 
-QWidget* GasTypeComboBoxItemDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
+QWidget *GasTypeComboBoxItemDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
 	// Create the combobox and populate it
 	QComboBox *cb = new QComboBox(parent);
@@ -85,7 +92,7 @@ QWidget* GasTypeComboBoxItemDelegate::createEditor(QWidget *parent, const QStyle
 
 void GasTypeComboBoxItemDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
 {
-	if(QComboBox *cb = qobject_cast<QComboBox *>(editor))
+	if (QComboBox *cb = qobject_cast<QComboBox *>(editor))
 		cb->setCurrentIndex(index.data(Qt::EditRole).toInt());
 	else
 		QStyledItemDelegate::setEditorData(editor, index);
@@ -94,25 +101,25 @@ void GasTypeComboBoxItemDelegate::setEditorData(QWidget *editor, const QModelInd
 
 void GasTypeComboBoxItemDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
 {
-	if(QComboBox *cb = qobject_cast<QComboBox *>(editor))
+	if (QComboBox *cb = qobject_cast<QComboBox *>(editor))
 		model->setData(index, cb->currentIndex(), Qt::EditRole);
 	else
 		QStyledItemDelegate::setModelData(editor, model, index);
 }
 
-ConfigureDiveComputerDialog::ConfigureDiveComputerDialog(QWidget *parent) :
-	QDialog(parent),
+ConfigureDiveComputerDialog::ConfigureDiveComputerDialog(QWidget *parent) : QDialog(parent),
 	config(0),
 	deviceDetails(0)
 {
 	ui.setupUi(this);
 
 	deviceDetails = new DeviceDetails(this);
-	config = new ConfigureDiveComputer(this);
+	config = new ConfigureDiveComputer();
+	connect(config, SIGNAL(progress(int)), ui.progressBar, SLOT(setValue(int)));
 	connect(config, SIGNAL(error(QString)), this, SLOT(configError(QString)));
 	connect(config, SIGNAL(message(QString)), this, SLOT(configMessage(QString)));
-	connect(config, SIGNAL(deviceDetailsChanged(DeviceDetails*)),
-		 this, SLOT(deviceDetailsReceived(DeviceDetails*)));
+	connect(config, SIGNAL(deviceDetailsChanged(DeviceDetails *)),
+		this, SLOT(deviceDetailsReceived(DeviceDetails *)));
 	connect(ui.retrieveDetails, SIGNAL(clicked()), this, SLOT(readSettings()));
 	connect(ui.resetButton, SIGNAL(clicked()), this, SLOT(resetSettings()));
 
@@ -141,21 +148,21 @@ ConfigureDiveComputerDialog::ConfigureDiveComputerDialog(QWidget *parent) :
 	settings.beginGroup("ConfigureDiveComputerDialog");
 	settings.beginGroup("ostc3GasTable");
 	for (int i = 0; i < ui.ostc3GasTable->columnCount(); i++) {
-        QVariant width = settings.value(QString("colwidth%1").arg(i));
+		QVariant width = settings.value(QString("colwidth%1").arg(i));
 		if (width.isValid())
 			ui.ostc3GasTable->setColumnWidth(i, width.toInt());
 	}
 	settings.endGroup();
 	settings.beginGroup("ostc3DilTable");
 	for (int i = 0; i < ui.ostc3DilTable->columnCount(); i++) {
-        QVariant width = settings.value(QString("colwidth%1").arg(i));
+		QVariant width = settings.value(QString("colwidth%1").arg(i));
 		if (width.isValid())
 			ui.ostc3DilTable->setColumnWidth(i, width.toInt());
 	}
 	settings.endGroup();
 	settings.beginGroup("ostc3SetPointTable");
 	for (int i = 0; i < ui.ostc3SetPointTable->columnCount(); i++) {
-        QVariant width = settings.value(QString("colwidth%1").arg(i));
+		QVariant width = settings.value(QString("colwidth%1").arg(i));
 		if (width.isValid())
 			ui.ostc3SetPointTable->setColumnWidth(i, width.toInt());
 	}
@@ -163,21 +170,21 @@ ConfigureDiveComputerDialog::ConfigureDiveComputerDialog(QWidget *parent) :
 
 	settings.beginGroup("ostcGasTable");
 	for (int i = 0; i < ui.ostcGasTable->columnCount(); i++) {
-        QVariant width = settings.value(QString("colwidth%1").arg(i));
+		QVariant width = settings.value(QString("colwidth%1").arg(i));
 		if (width.isValid())
 			ui.ostcGasTable->setColumnWidth(i, width.toInt());
 	}
 	settings.endGroup();
 	settings.beginGroup("ostcDilTable");
 	for (int i = 0; i < ui.ostcDilTable->columnCount(); i++) {
-        QVariant width = settings.value(QString("colwidth%1").arg(i));
+		QVariant width = settings.value(QString("colwidth%1").arg(i));
 		if (width.isValid())
 			ui.ostcDilTable->setColumnWidth(i, width.toInt());
 	}
 	settings.endGroup();
 	settings.beginGroup("ostcSetPointTable");
 	for (int i = 0; i < ui.ostcSetPointTable->columnCount(); i++) {
-        QVariant width = settings.value(QString("colwidth%1").arg(i));
+		QVariant width = settings.value(QString("colwidth%1").arg(i));
 		if (width.isValid())
 			ui.ostcSetPointTable->setColumnWidth(i, width.toInt());
 	}
@@ -185,6 +192,106 @@ ConfigureDiveComputerDialog::ConfigureDiveComputerDialog(QWidget *parent) :
 	settings.endGroup();
 }
 
+OstcFirmwareCheck::OstcFirmwareCheck(QString product)
+{
+	QUrl url;
+	if (product == "OSTC 3") {
+		url = QUrl("http://www.heinrichsweikamp.net/autofirmware/ostc3_changelog.txt");
+		latestFirmwareHexFile = QString("http://www.heinrichsweikamp.net/autofirmware/ostc3_firmware.hex");
+	} else if (product == "OSTC Sport") {
+		url = QUrl("http://www.heinrichsweikamp.net/autofirmware/ostc_sport_changelog.txt");
+		latestFirmwareHexFile = QString("http://www.heinrichsweikamp.net/autofirmware/ostc_sport_firmware.hex");
+	} else { // not one of the known dive computers
+		return;
+	}
+	connect(&manager, SIGNAL(finished(QNetworkReply *)), this, SLOT(parseOstcFwVersion(QNetworkReply *)));
+	QNetworkRequest download(url);
+	manager.get(download);
+}
+
+void OstcFirmwareCheck::parseOstcFwVersion(QNetworkReply *reply)
+{
+	QString parse = reply->readAll();
+	int firstOpenBracket = parse.indexOf('[');
+	int firstCloseBracket = parse.indexOf(']');
+	latestFirmwareAvailable = parse.mid(firstOpenBracket + 1, firstCloseBracket - firstOpenBracket - 1);
+	disconnect(&manager, SIGNAL(finished(QNetworkReply *)), this, SLOT(parseOstcFwVersion(QNetworkReply *)));
+}
+
+void OstcFirmwareCheck::checkLatest(QWidget *_parent, device_data_t *data)
+{
+	devData = *data;
+	parent = _parent;
+	// If we didn't find a current firmware version stop this hole thing here.
+	if (latestFirmwareAvailable.isEmpty())
+		return;
+
+	// for now libdivecomputer gives us the firmware on device undecoded as integer
+	// for the OSTC that means highbyte.lowbyte is the version number
+	int firmwareOnDevice = devData.libdc_firmware;
+	QString firmwareOnDeviceString = QString("%1.%2").arg(firmwareOnDevice / 256).arg(firmwareOnDevice % 256);
+
+	// Convert the latestFirmwareAvailable to a integear we can compare with
+	QStringList fwParts = latestFirmwareAvailable.split(".");
+	int latestFirmwareAvailableNumber = fwParts[0].toInt() * 256 + fwParts[1].toInt();
+	if (latestFirmwareAvailableNumber > firmwareOnDevice) {
+		QMessageBox response(parent);
+		QString message = tr("You should update the firmware on your dive computer: you have version %1 but the latest stable version is %2")
+					  .arg(firmwareOnDeviceString)
+					  .arg(latestFirmwareAvailable);
+		if (strcmp(data->product, "OSTC Sport") == 0)
+			message += tr("\n\nPlease start Bluetooth on your OSTC Sport and do the same preparations as for a logbook download before continuing with the update");
+		response.addButton(tr("Not now"), QMessageBox::RejectRole);
+		response.addButton(tr("Update firmware"), QMessageBox::AcceptRole);
+		response.setText(message);
+		response.setWindowTitle(tr("Firmware upgrade notice"));
+		response.setIcon(QMessageBox::Question);
+		response.setWindowModality(Qt::WindowModal);
+		int ret = response.exec();
+		if (ret == QMessageBox::Accepted)
+			upgradeFirmware();
+	}
+}
+
+void OstcFirmwareCheck::upgradeFirmware()
+{
+	// start download of latestFirmwareHexFile
+	QString saveFileName = latestFirmwareHexFile;
+	saveFileName.replace("http://www.heinrichsweikamp.net/autofirmware/", "");
+	saveFileName.replace("firmware", latestFirmwareAvailable);
+	QString filename = existing_filename ?: prefs.default_filename;
+	QFileInfo fi(filename);
+	filename = fi.absolutePath().append(QDir::separator()).append(saveFileName);
+	storeFirmware = QFileDialog::getSaveFileName(parent, tr("Save the downloaded firmware as"),
+						     filename, tr("HEX files (*.hex)"));
+	if (storeFirmware.isEmpty())
+		return;
+
+	connect(&manager, SIGNAL(finished(QNetworkReply *)), this, SLOT(saveOstcFirmware(QNetworkReply *)));
+	QNetworkRequest download(latestFirmwareHexFile);
+	manager.get(download);
+}
+
+void OstcFirmwareCheck::saveOstcFirmware(QNetworkReply *reply)
+{
+	// firmware is downloaded
+	// call config->startFirmwareUpdate() with that file and the device data
+
+	QByteArray firmwareData = reply->readAll();
+	QFile file(storeFirmware);
+	file.open(QIODevice::WriteOnly);
+	file.write(firmwareData);
+	file.close();
+	QProgressDialog *dialog = new QProgressDialog("Updating firmware", "", 0, 100);
+	dialog->setCancelButton(0);
+	dialog->setAutoClose(true);
+	ConfigureDiveComputer *config = new ConfigureDiveComputer();
+	connect(config, SIGNAL(message(QString)), dialog, SLOT(setLabelText(QString)));
+	connect(config, SIGNAL(error(QString)), dialog, SLOT(setLabelText(QString)));
+	connect(config, SIGNAL(progress(int)), dialog, SLOT(setValue(int)));
+	config->startFirmwareUpdate(storeFirmware, &devData);
+}
+
 ConfigureDiveComputerDialog::~ConfigureDiveComputerDialog()
 {
 	QSettings settings;
@@ -274,7 +381,7 @@ void ConfigureDiveComputerDialog::fill_computer_list()
 
 void ConfigureDiveComputerDialog::populateDeviceDetails()
 {
-	switch(ui.dcStackedWidget->currentIndex()) {
+	switch (ui.dcStackedWidget->currentIndex()) {
 	case 0:
 		populateDeviceDetailsOSTC3();
 		break;
@@ -374,7 +481,7 @@ void ConfigureDiveComputerDialog::populateDeviceDetailsOSTC3()
 	dil2.oxygen = GET_INT_FROM(ui.ostc3DilTable->item(1, 1), 21);
 	dil2.helium = GET_INT_FROM(ui.ostc3DilTable->item(1, 2), 0);
 	dil2.type = GET_INT_FROM(ui.ostc3DilTable->item(1, 3), 0);
-	dil2.depth = GET_INT_FROM(ui.ostc3DilTable->item(1, 4),0);
+	dil2.depth = GET_INT_FROM(ui.ostc3DilTable->item(1, 4), 0);
 
 	dil3.oxygen = GET_INT_FROM(ui.ostc3DilTable->item(2, 1), 21);
 	dil3.helium = GET_INT_FROM(ui.ostc3DilTable->item(2, 2), 0);
@@ -500,7 +607,7 @@ void ConfigureDiveComputerDialog::populateDeviceDetailsOSTC()
 	dil2.oxygen = GET_INT_FROM(ui.ostcDilTable->item(1, 1), 21);
 	dil2.helium = GET_INT_FROM(ui.ostcDilTable->item(1, 2), 0);
 	dil2.type = GET_INT_FROM(ui.ostcDilTable->item(1, 3), 0);
-	dil2.depth = GET_INT_FROM(ui.ostcDilTable->item(1, 4),0);
+	dil2.depth = GET_INT_FROM(ui.ostcDilTable->item(1, 4), 0);
 
 	dil3.oxygen = GET_INT_FROM(ui.ostcDilTable->item(2, 1), 21);
 	dil3.helium = GET_INT_FROM(ui.ostcDilTable->item(2, 2), 0);
@@ -571,8 +678,9 @@ void ConfigureDiveComputerDialog::populateDeviceDetailsSuuntoVyper()
 
 void ConfigureDiveComputerDialog::readSettings()
 {
-	ui.statusLabel->clear();
-	ui.errorLabel->clear();
+	ui.progressBar->setValue(0);
+	ui.progressBar->setFormat("%p%");
+	ui.progressBar->setTextVisible(true);
 
 	getDeviceData();
 	config->readSettings(&device_data);
@@ -580,8 +688,9 @@ void ConfigureDiveComputerDialog::readSettings()
 
 void ConfigureDiveComputerDialog::resetSettings()
 {
-	ui.statusLabel->clear();
-	ui.errorLabel->clear();
+	ui.progressBar->setValue(0);
+	ui.progressBar->setFormat("%p%");
+	ui.progressBar->setTextVisible(true);
 
 	getDeviceData();
 	config->resetSettings(&device_data);
@@ -589,13 +698,12 @@ void ConfigureDiveComputerDialog::resetSettings()
 
 void ConfigureDiveComputerDialog::configMessage(QString msg)
 {
-	ui.statusLabel->setText(msg);
+	ui.progressBar->setFormat(msg);
 }
 
 void ConfigureDiveComputerDialog::configError(QString err)
 {
-	ui.statusLabel->setText("");
-	ui.errorLabel->setText(err);
+	ui.progressBar->setFormat("Error: " + err);
 }
 
 void ConfigureDiveComputerDialog::getDeviceData()
@@ -617,6 +725,10 @@ void ConfigureDiveComputerDialog::on_cancel_clicked()
 
 void ConfigureDiveComputerDialog::on_saveSettingsPushButton_clicked()
 {
+	ui.progressBar->setValue(0);
+	ui.progressBar->setFormat("%p%");
+	ui.progressBar->setTextVisible(true);
+
 	populateDeviceDetails();
 	getDeviceData();
 	config->saveDeviceDetails(deviceDetails, &device_data);
@@ -634,7 +746,7 @@ void ConfigureDiveComputerDialog::reloadValues()
 	ui.saveSettingsPushButton->setEnabled(true);
 	ui.backupButton->setEnabled(true);
 
-	switch(ui.dcStackedWidget->currentIndex()) {
+	switch (ui.dcStackedWidget->currentIndex()) {
 	case 0:
 		reloadValuesOSTC3();
 		break;
@@ -681,64 +793,64 @@ void ConfigureDiveComputerDialog::reloadValuesOSTC3()
 	ui.setPointFallbackCheckBox->setChecked(deviceDetails->setPointFallback());
 
 	//load gas 1 values
-	ui.ostc3GasTable->setItem(0,1, new QTableWidgetItem(QString::number(deviceDetails->gas1().oxygen)));
-	ui.ostc3GasTable->setItem(0,2, new QTableWidgetItem(QString::number(deviceDetails->gas1().helium)));
-	ui.ostc3GasTable->setItem(0,3, new QTableWidgetItem(QString::number(deviceDetails->gas1().type)));
-	ui.ostc3GasTable->setItem(0,4, new QTableWidgetItem(QString::number(deviceDetails->gas1().depth)));
+	ui.ostc3GasTable->setItem(0, 1, new QTableWidgetItem(QString::number(deviceDetails->gas1().oxygen)));
+	ui.ostc3GasTable->setItem(0, 2, new QTableWidgetItem(QString::number(deviceDetails->gas1().helium)));
+	ui.ostc3GasTable->setItem(0, 3, new QTableWidgetItem(QString::number(deviceDetails->gas1().type)));
+	ui.ostc3GasTable->setItem(0, 4, new QTableWidgetItem(QString::number(deviceDetails->gas1().depth)));
 
 	//load gas 2 values
-	ui.ostc3GasTable->setItem(1,1, new QTableWidgetItem(QString::number(deviceDetails->gas2().oxygen)));
-	ui.ostc3GasTable->setItem(1,2, new QTableWidgetItem(QString::number(deviceDetails->gas2().helium)));
-	ui.ostc3GasTable->setItem(1,3, new QTableWidgetItem(QString::number(deviceDetails->gas2().type)));
-	ui.ostc3GasTable->setItem(1,4, new QTableWidgetItem(QString::number(deviceDetails->gas2().depth)));
+	ui.ostc3GasTable->setItem(1, 1, new QTableWidgetItem(QString::number(deviceDetails->gas2().oxygen)));
+	ui.ostc3GasTable->setItem(1, 2, new QTableWidgetItem(QString::number(deviceDetails->gas2().helium)));
+	ui.ostc3GasTable->setItem(1, 3, new QTableWidgetItem(QString::number(deviceDetails->gas2().type)));
+	ui.ostc3GasTable->setItem(1, 4, new QTableWidgetItem(QString::number(deviceDetails->gas2().depth)));
 
 	//load gas 3 values
-	ui.ostc3GasTable->setItem(2,1, new QTableWidgetItem(QString::number(deviceDetails->gas3().oxygen)));
-	ui.ostc3GasTable->setItem(2,2, new QTableWidgetItem(QString::number(deviceDetails->gas3().helium)));
-	ui.ostc3GasTable->setItem(2,3, new QTableWidgetItem(QString::number(deviceDetails->gas3().type)));
-	ui.ostc3GasTable->setItem(2,4, new QTableWidgetItem(QString::number(deviceDetails->gas3().depth)));
+	ui.ostc3GasTable->setItem(2, 1, new QTableWidgetItem(QString::number(deviceDetails->gas3().oxygen)));
+	ui.ostc3GasTable->setItem(2, 2, new QTableWidgetItem(QString::number(deviceDetails->gas3().helium)));
+	ui.ostc3GasTable->setItem(2, 3, new QTableWidgetItem(QString::number(deviceDetails->gas3().type)));
+	ui.ostc3GasTable->setItem(2, 4, new QTableWidgetItem(QString::number(deviceDetails->gas3().depth)));
 
 	//load gas 4 values
-	ui.ostc3GasTable->setItem(3,1, new QTableWidgetItem(QString::number(deviceDetails->gas4().oxygen)));
-	ui.ostc3GasTable->setItem(3,2, new QTableWidgetItem(QString::number(deviceDetails->gas4().helium)));
-	ui.ostc3GasTable->setItem(3,3, new QTableWidgetItem(QString::number(deviceDetails->gas4().type)));
-	ui.ostc3GasTable->setItem(3,4, new QTableWidgetItem(QString::number(deviceDetails->gas4().depth)));
+	ui.ostc3GasTable->setItem(3, 1, new QTableWidgetItem(QString::number(deviceDetails->gas4().oxygen)));
+	ui.ostc3GasTable->setItem(3, 2, new QTableWidgetItem(QString::number(deviceDetails->gas4().helium)));
+	ui.ostc3GasTable->setItem(3, 3, new QTableWidgetItem(QString::number(deviceDetails->gas4().type)));
+	ui.ostc3GasTable->setItem(3, 4, new QTableWidgetItem(QString::number(deviceDetails->gas4().depth)));
 
 	//load gas 5 values
-	ui.ostc3GasTable->setItem(4,1, new QTableWidgetItem(QString::number(deviceDetails->gas5().oxygen)));
-	ui.ostc3GasTable->setItem(4,2, new QTableWidgetItem(QString::number(deviceDetails->gas5().helium)));
-	ui.ostc3GasTable->setItem(4,3, new QTableWidgetItem(QString::number(deviceDetails->gas5().type)));
-	ui.ostc3GasTable->setItem(4,4, new QTableWidgetItem(QString::number(deviceDetails->gas5().depth)));
+	ui.ostc3GasTable->setItem(4, 1, new QTableWidgetItem(QString::number(deviceDetails->gas5().oxygen)));
+	ui.ostc3GasTable->setItem(4, 2, new QTableWidgetItem(QString::number(deviceDetails->gas5().helium)));
+	ui.ostc3GasTable->setItem(4, 3, new QTableWidgetItem(QString::number(deviceDetails->gas5().type)));
+	ui.ostc3GasTable->setItem(4, 4, new QTableWidgetItem(QString::number(deviceDetails->gas5().depth)));
 
 	//load dil 1 values
-	ui.ostc3DilTable->setItem(0,1, new QTableWidgetItem(QString::number(deviceDetails->dil1().oxygen)));
-	ui.ostc3DilTable->setItem(0,2, new QTableWidgetItem(QString::number(deviceDetails->dil1().helium)));
-	ui.ostc3DilTable->setItem(0,3, new QTableWidgetItem(QString::number(deviceDetails->dil1().type)));
-	ui.ostc3DilTable->setItem(0,4, new QTableWidgetItem(QString::number(deviceDetails->dil1().depth)));
+	ui.ostc3DilTable->setItem(0, 1, new QTableWidgetItem(QString::number(deviceDetails->dil1().oxygen)));
+	ui.ostc3DilTable->setItem(0, 2, new QTableWidgetItem(QString::number(deviceDetails->dil1().helium)));
+	ui.ostc3DilTable->setItem(0, 3, new QTableWidgetItem(QString::number(deviceDetails->dil1().type)));
+	ui.ostc3DilTable->setItem(0, 4, new QTableWidgetItem(QString::number(deviceDetails->dil1().depth)));
 
 	//load dil 2 values
-	ui.ostc3DilTable->setItem(1,1, new QTableWidgetItem(QString::number(deviceDetails->dil2().oxygen)));
-	ui.ostc3DilTable->setItem(1,2, new QTableWidgetItem(QString::number(deviceDetails->dil2().helium)));
-	ui.ostc3DilTable->setItem(1,3, new QTableWidgetItem(QString::number(deviceDetails->dil2().type)));
-	ui.ostc3DilTable->setItem(1,4, new QTableWidgetItem(QString::number(deviceDetails->dil2().depth)));
+	ui.ostc3DilTable->setItem(1, 1, new QTableWidgetItem(QString::number(deviceDetails->dil2().oxygen)));
+	ui.ostc3DilTable->setItem(1, 2, new QTableWidgetItem(QString::number(deviceDetails->dil2().helium)));
+	ui.ostc3DilTable->setItem(1, 3, new QTableWidgetItem(QString::number(deviceDetails->dil2().type)));
+	ui.ostc3DilTable->setItem(1, 4, new QTableWidgetItem(QString::number(deviceDetails->dil2().depth)));
 
 	//load dil 3 values
-	ui.ostc3DilTable->setItem(2,1, new QTableWidgetItem(QString::number(deviceDetails->dil3().oxygen)));
-	ui.ostc3DilTable->setItem(2,2, new QTableWidgetItem(QString::number(deviceDetails->dil3().helium)));
-	ui.ostc3DilTable->setItem(2,3, new QTableWidgetItem(QString::number(deviceDetails->dil3().type)));
-	ui.ostc3DilTable->setItem(2,4, new QTableWidgetItem(QString::number(deviceDetails->dil3().depth)));
+	ui.ostc3DilTable->setItem(2, 1, new QTableWidgetItem(QString::number(deviceDetails->dil3().oxygen)));
+	ui.ostc3DilTable->setItem(2, 2, new QTableWidgetItem(QString::number(deviceDetails->dil3().helium)));
+	ui.ostc3DilTable->setItem(2, 3, new QTableWidgetItem(QString::number(deviceDetails->dil3().type)));
+	ui.ostc3DilTable->setItem(2, 4, new QTableWidgetItem(QString::number(deviceDetails->dil3().depth)));
 
 	//load dil 4 values
-	ui.ostc3DilTable->setItem(3,1, new QTableWidgetItem(QString::number(deviceDetails->dil4().oxygen)));
-	ui.ostc3DilTable->setItem(3,2, new QTableWidgetItem(QString::number(deviceDetails->dil4().helium)));
-	ui.ostc3DilTable->setItem(3,3, new QTableWidgetItem(QString::number(deviceDetails->dil4().type)));
-	ui.ostc3DilTable->setItem(3,4, new QTableWidgetItem(QString::number(deviceDetails->dil4().depth)));
+	ui.ostc3DilTable->setItem(3, 1, new QTableWidgetItem(QString::number(deviceDetails->dil4().oxygen)));
+	ui.ostc3DilTable->setItem(3, 2, new QTableWidgetItem(QString::number(deviceDetails->dil4().helium)));
+	ui.ostc3DilTable->setItem(3, 3, new QTableWidgetItem(QString::number(deviceDetails->dil4().type)));
+	ui.ostc3DilTable->setItem(3, 4, new QTableWidgetItem(QString::number(deviceDetails->dil4().depth)));
 
 	//load dil 5 values
-	ui.ostc3DilTable->setItem(4,1, new QTableWidgetItem(QString::number(deviceDetails->dil5().oxygen)));
-	ui.ostc3DilTable->setItem(4,2, new QTableWidgetItem(QString::number(deviceDetails->dil5().helium)));
-	ui.ostc3DilTable->setItem(4,3, new QTableWidgetItem(QString::number(deviceDetails->dil5().type)));
-	ui.ostc3DilTable->setItem(4,4, new QTableWidgetItem(QString::number(deviceDetails->dil5().depth)));
+	ui.ostc3DilTable->setItem(4, 1, new QTableWidgetItem(QString::number(deviceDetails->dil5().oxygen)));
+	ui.ostc3DilTable->setItem(4, 2, new QTableWidgetItem(QString::number(deviceDetails->dil5().helium)));
+	ui.ostc3DilTable->setItem(4, 3, new QTableWidgetItem(QString::number(deviceDetails->dil5().type)));
+	ui.ostc3DilTable->setItem(4, 4, new QTableWidgetItem(QString::number(deviceDetails->dil5().depth)));
 
 	//load set point 1 values
 	ui.ostc3SetPointTable->setItem(0, 1, new QTableWidgetItem(QString::number(deviceDetails->sp1().sp)));
@@ -763,7 +875,7 @@ void ConfigureDiveComputerDialog::reloadValuesOSTC3()
 
 void ConfigureDiveComputerDialog::reloadValuesOSTC()
 {
-/*
+	/*
 # Not in OSTC
 setBrightness
 setCalibrationGas
@@ -786,7 +898,7 @@ setNumberOfDives
 	ui.desaturationSpinBox_3->setValue(deviceDetails->desaturation());
 	ui.lastDecoSpinBox_3->setValue(deviceDetails->lastDeco());
 	ui.samplingRateSpinBox_3->setValue(deviceDetails->samplingRate());
-	ui.salinityDoubleSpinBox_3->setValue((double) deviceDetails->salinity() / 100.0);
+	ui.salinityDoubleSpinBox_3->setValue((double)deviceDetails->salinity() / 100.0);
 	ui.dateFormatComboBox_3->setCurrentIndex(deviceDetails->dateFormat());
 	ui.safetyStopCheckBox_3->setChecked(deviceDetails->safetyStop());
 	ui.gfHighSpinBox_3->setValue(deviceDetails->gfHigh());
@@ -801,64 +913,64 @@ setNumberOfDives
 	ui.numberOfDivesSpinBox_3->setValue(deviceDetails->numberOfDives());
 
 	//load gas 1 values
-	ui.ostcGasTable->setItem(0,1, new QTableWidgetItem(QString::number(deviceDetails->gas1().oxygen)));
-	ui.ostcGasTable->setItem(0,2, new QTableWidgetItem(QString::number(deviceDetails->gas1().helium)));
-	ui.ostcGasTable->setItem(0,3, new QTableWidgetItem(QString::number(deviceDetails->gas1().type)));
-	ui.ostcGasTable->setItem(0,4, new QTableWidgetItem(QString::number(deviceDetails->gas1().depth)));
+	ui.ostcGasTable->setItem(0, 1, new QTableWidgetItem(QString::number(deviceDetails->gas1().oxygen)));
+	ui.ostcGasTable->setItem(0, 2, new QTableWidgetItem(QString::number(deviceDetails->gas1().helium)));
+	ui.ostcGasTable->setItem(0, 3, new QTableWidgetItem(QString::number(deviceDetails->gas1().type)));
+	ui.ostcGasTable->setItem(0, 4, new QTableWidgetItem(QString::number(deviceDetails->gas1().depth)));
 
 	//load gas 2 values
-	ui.ostcGasTable->setItem(1,1, new QTableWidgetItem(QString::number(deviceDetails->gas2().oxygen)));
-	ui.ostcGasTable->setItem(1,2, new QTableWidgetItem(QString::number(deviceDetails->gas2().helium)));
-	ui.ostcGasTable->setItem(1,3, new QTableWidgetItem(QString::number(deviceDetails->gas2().type)));
-	ui.ostcGasTable->setItem(1,4, new QTableWidgetItem(QString::number(deviceDetails->gas2().depth)));
+	ui.ostcGasTable->setItem(1, 1, new QTableWidgetItem(QString::number(deviceDetails->gas2().oxygen)));
+	ui.ostcGasTable->setItem(1, 2, new QTableWidgetItem(QString::number(deviceDetails->gas2().helium)));
+	ui.ostcGasTable->setItem(1, 3, new QTableWidgetItem(QString::number(deviceDetails->gas2().type)));
+	ui.ostcGasTable->setItem(1, 4, new QTableWidgetItem(QString::number(deviceDetails->gas2().depth)));
 
 	//load gas 3 values
-	ui.ostcGasTable->setItem(2,1, new QTableWidgetItem(QString::number(deviceDetails->gas3().oxygen)));
-	ui.ostcGasTable->setItem(2,2, new QTableWidgetItem(QString::number(deviceDetails->gas3().helium)));
-	ui.ostcGasTable->setItem(2,3, new QTableWidgetItem(QString::number(deviceDetails->gas3().type)));
-	ui.ostcGasTable->setItem(2,4, new QTableWidgetItem(QString::number(deviceDetails->gas3().depth)));
+	ui.ostcGasTable->setItem(2, 1, new QTableWidgetItem(QString::number(deviceDetails->gas3().oxygen)));
+	ui.ostcGasTable->setItem(2, 2, new QTableWidgetItem(QString::number(deviceDetails->gas3().helium)));
+	ui.ostcGasTable->setItem(2, 3, new QTableWidgetItem(QString::number(deviceDetails->gas3().type)));
+	ui.ostcGasTable->setItem(2, 4, new QTableWidgetItem(QString::number(deviceDetails->gas3().depth)));
 
 	//load gas 4 values
-	ui.ostcGasTable->setItem(3,1, new QTableWidgetItem(QString::number(deviceDetails->gas4().oxygen)));
-	ui.ostcGasTable->setItem(3,2, new QTableWidgetItem(QString::number(deviceDetails->gas4().helium)));
-	ui.ostcGasTable->setItem(3,3, new QTableWidgetItem(QString::number(deviceDetails->gas4().type)));
-	ui.ostcGasTable->setItem(3,4, new QTableWidgetItem(QString::number(deviceDetails->gas4().depth)));
+	ui.ostcGasTable->setItem(3, 1, new QTableWidgetItem(QString::number(deviceDetails->gas4().oxygen)));
+	ui.ostcGasTable->setItem(3, 2, new QTableWidgetItem(QString::number(deviceDetails->gas4().helium)));
+	ui.ostcGasTable->setItem(3, 3, new QTableWidgetItem(QString::number(deviceDetails->gas4().type)));
+	ui.ostcGasTable->setItem(3, 4, new QTableWidgetItem(QString::number(deviceDetails->gas4().depth)));
 
 	//load gas 5 values
-	ui.ostcGasTable->setItem(4,1, new QTableWidgetItem(QString::number(deviceDetails->gas5().oxygen)));
-	ui.ostcGasTable->setItem(4,2, new QTableWidgetItem(QString::number(deviceDetails->gas5().helium)));
-	ui.ostcGasTable->setItem(4,3, new QTableWidgetItem(QString::number(deviceDetails->gas5().type)));
-	ui.ostcGasTable->setItem(4,4, new QTableWidgetItem(QString::number(deviceDetails->gas5().depth)));
+	ui.ostcGasTable->setItem(4, 1, new QTableWidgetItem(QString::number(deviceDetails->gas5().oxygen)));
+	ui.ostcGasTable->setItem(4, 2, new QTableWidgetItem(QString::number(deviceDetails->gas5().helium)));
+	ui.ostcGasTable->setItem(4, 3, new QTableWidgetItem(QString::number(deviceDetails->gas5().type)));
+	ui.ostcGasTable->setItem(4, 4, new QTableWidgetItem(QString::number(deviceDetails->gas5().depth)));
 
 	//load dil 1 values
-	ui.ostcDilTable->setItem(0,1, new QTableWidgetItem(QString::number(deviceDetails->dil1().oxygen)));
-	ui.ostcDilTable->setItem(0,2, new QTableWidgetItem(QString::number(deviceDetails->dil1().helium)));
-	ui.ostcDilTable->setItem(0,3, new QTableWidgetItem(QString::number(deviceDetails->dil1().type)));
-	ui.ostcDilTable->setItem(0,4, new QTableWidgetItem(QString::number(deviceDetails->dil1().depth)));
+	ui.ostcDilTable->setItem(0, 1, new QTableWidgetItem(QString::number(deviceDetails->dil1().oxygen)));
+	ui.ostcDilTable->setItem(0, 2, new QTableWidgetItem(QString::number(deviceDetails->dil1().helium)));
+	ui.ostcDilTable->setItem(0, 3, new QTableWidgetItem(QString::number(deviceDetails->dil1().type)));
+	ui.ostcDilTable->setItem(0, 4, new QTableWidgetItem(QString::number(deviceDetails->dil1().depth)));
 
 	//load dil 2 values
-	ui.ostcDilTable->setItem(1,1, new QTableWidgetItem(QString::number(deviceDetails->dil2().oxygen)));
-	ui.ostcDilTable->setItem(1,2, new QTableWidgetItem(QString::number(deviceDetails->dil2().helium)));
-	ui.ostcDilTable->setItem(1,3, new QTableWidgetItem(QString::number(deviceDetails->dil2().type)));
-	ui.ostcDilTable->setItem(1,4, new QTableWidgetItem(QString::number(deviceDetails->dil2().depth)));
+	ui.ostcDilTable->setItem(1, 1, new QTableWidgetItem(QString::number(deviceDetails->dil2().oxygen)));
+	ui.ostcDilTable->setItem(1, 2, new QTableWidgetItem(QString::number(deviceDetails->dil2().helium)));
+	ui.ostcDilTable->setItem(1, 3, new QTableWidgetItem(QString::number(deviceDetails->dil2().type)));
+	ui.ostcDilTable->setItem(1, 4, new QTableWidgetItem(QString::number(deviceDetails->dil2().depth)));
 
 	//load dil 3 values
-	ui.ostcDilTable->setItem(2,1, new QTableWidgetItem(QString::number(deviceDetails->dil3().oxygen)));
-	ui.ostcDilTable->setItem(2,2, new QTableWidgetItem(QString::number(deviceDetails->dil3().helium)));
-	ui.ostcDilTable->setItem(2,3, new QTableWidgetItem(QString::number(deviceDetails->dil3().type)));
-	ui.ostcDilTable->setItem(2,4, new QTableWidgetItem(QString::number(deviceDetails->dil3().depth)));
+	ui.ostcDilTable->setItem(2, 1, new QTableWidgetItem(QString::number(deviceDetails->dil3().oxygen)));
+	ui.ostcDilTable->setItem(2, 2, new QTableWidgetItem(QString::number(deviceDetails->dil3().helium)));
+	ui.ostcDilTable->setItem(2, 3, new QTableWidgetItem(QString::number(deviceDetails->dil3().type)));
+	ui.ostcDilTable->setItem(2, 4, new QTableWidgetItem(QString::number(deviceDetails->dil3().depth)));
 
 	//load dil 4 values
-	ui.ostcDilTable->setItem(3,1, new QTableWidgetItem(QString::number(deviceDetails->dil4().oxygen)));
-	ui.ostcDilTable->setItem(3,2, new QTableWidgetItem(QString::number(deviceDetails->dil4().helium)));
-	ui.ostcDilTable->setItem(3,3, new QTableWidgetItem(QString::number(deviceDetails->dil4().type)));
-	ui.ostcDilTable->setItem(3,4, new QTableWidgetItem(QString::number(deviceDetails->dil4().depth)));
+	ui.ostcDilTable->setItem(3, 1, new QTableWidgetItem(QString::number(deviceDetails->dil4().oxygen)));
+	ui.ostcDilTable->setItem(3, 2, new QTableWidgetItem(QString::number(deviceDetails->dil4().helium)));
+	ui.ostcDilTable->setItem(3, 3, new QTableWidgetItem(QString::number(deviceDetails->dil4().type)));
+	ui.ostcDilTable->setItem(3, 4, new QTableWidgetItem(QString::number(deviceDetails->dil4().depth)));
 
 	//load dil 5 values
-	ui.ostcDilTable->setItem(4,1, new QTableWidgetItem(QString::number(deviceDetails->dil5().oxygen)));
-	ui.ostcDilTable->setItem(4,2, new QTableWidgetItem(QString::number(deviceDetails->dil5().helium)));
-	ui.ostcDilTable->setItem(4,3, new QTableWidgetItem(QString::number(deviceDetails->dil5().type)));
-	ui.ostcDilTable->setItem(4,4, new QTableWidgetItem(QString::number(deviceDetails->dil5().depth)));
+	ui.ostcDilTable->setItem(4, 1, new QTableWidgetItem(QString::number(deviceDetails->dil5().oxygen)));
+	ui.ostcDilTable->setItem(4, 2, new QTableWidgetItem(QString::number(deviceDetails->dil5().helium)));
+	ui.ostcDilTable->setItem(4, 3, new QTableWidgetItem(QString::number(deviceDetails->dil5().type)));
+	ui.ostcDilTable->setItem(4, 4, new QTableWidgetItem(QString::number(deviceDetails->dil5().depth)));
 
 	//load set point 1 values
 	ui.ostcSetPointTable->setItem(0, 1, new QTableWidgetItem(QString::number(deviceDetails->sp1().sp)));
@@ -913,21 +1025,18 @@ void ConfigureDiveComputerDialog::on_backupButton_clicked()
 	QFileInfo fi(filename);
 	filename = fi.absolutePath().append(QDir::separator()).append("Backup.xml");
 	QString backupPath = QFileDialog::getSaveFileName(this, tr("Backup dive computer settings"),
-							  filename, tr("Backup files (*.xml)")
-							  );
+							  filename, tr("Backup files (*.xml)"));
 	if (!backupPath.isEmpty()) {
 		populateDeviceDetails();
 		getDeviceData();
 		if (!config->saveXMLBackup(backupPath, deviceDetails, &device_data)) {
 			QMessageBox::critical(this, tr("XML backup error"),
 					      tr("An error occurred while saving the backup file.\n%1")
-					      .arg(config->lastError)
-					      );
+						      .arg(config->lastError));
 		} else {
 			QMessageBox::information(this, tr("Backup succeeded"),
 						 tr("Your settings have been saved to: %1")
-						 .arg(backupPath)
-						 );
+							 .arg(backupPath));
 		}
 	}
 }
@@ -938,21 +1047,18 @@ void ConfigureDiveComputerDialog::on_restoreBackupButton_clicked()
 	QFileInfo fi(filename);
 	filename = fi.absolutePath().append(QDir::separator()).append("Backup.xml");
 	QString restorePath = QFileDialog::getOpenFileName(this, tr("Restore dive computer settings"),
-							   filename, tr("Backup files (*.xml)")
-							   );
+							   filename, tr("Backup files (*.xml)"));
 	if (!restorePath.isEmpty()) {
 		if (!config->restoreXMLBackup(restorePath, deviceDetails)) {
 			QMessageBox::critical(this, tr("XML restore error"),
 					      tr("An error occurred while restoring the backup file.\n%1")
-					      .arg(config->lastError)
-					      );
+						      .arg(config->lastError));
 		} else {
 			reloadValues();
 			//getDeviceData();
 			//config->saveDeviceDetails(deviceDetails, &device_data);
 			QMessageBox::information(this, tr("Restore succeeded"),
-						 tr("Your settings have been restored successfully.")
-						 );
+						 tr("Your settings have been restored successfully."));
 		}
 	}
 }
@@ -963,14 +1069,18 @@ void ConfigureDiveComputerDialog::on_updateFirmwareButton_clicked()
 	QFileInfo fi(filename);
 	filename = fi.absolutePath();
 	QString firmwarePath = QFileDialog::getOpenFileName(this, tr("Select firmware file"),
-							  filename, tr("All files (*.*)")
-							  );
+							    filename, tr("All files (*.*)"));
 	if (!firmwarePath.isEmpty()) {
+		ui.progressBar->setValue(0);
+		ui.progressBar->setFormat("%p%");
+		ui.progressBar->setTextVisible(true);
+
 		getDeviceData();
 		config->startFirmwareUpdate(firmwarePath, &device_data);
 	}
 }
 
+
 void ConfigureDiveComputerDialog::on_DiveComputerList_currentRowChanged(int currentRow)
 {
 	// Disable the buttons to do operations on this data
@@ -981,7 +1091,7 @@ void ConfigureDiveComputerDialog::on_DiveComputerList_currentRowChanged(int curr
 	case 0:
 		selected_vendor = "Heinrichs Weikamp";
 		selected_product = "OSTC 3";
-		ui.updateFirmwareButton->setEnabled(false);
+		ui.updateFirmwareButton->setEnabled(true);
 		break;
 	case 1:
 		selected_vendor = "Suunto";
