@@ -5,29 +5,12 @@
  *
  */
 #include "divelistview.h"
-#include "models.h"
 #include "filtermodels.h"
 #include "modeldelegates.h"
 #include "mainwindow.h"
-#include "subsurfacewebservices.h"
-#include "divelogexportdialog.h"
-#include "../display.h"
-#include "exif.h"
-#include "../file.h"
-#include <QApplication>
-#include <QHeaderView>
-#include <QDebug>
 #include <QSettings>
-#include <QKeyEvent>
-#include <QSortFilterProxyModel>
-#include <QAction>
-#include <QLineEdit>
-#include <QKeyEvent>
-#include <QMenu>
 #include <QFileDialog>
-#include <string>
-#include <iostream>
-#include "../qthelper.h"
+#include "qthelper.h"
 
 //                                #  Date  Rtg Dpth  Dur  Tmp Wght Suit  Cyl  Gas  SAC  OTU  CNS  Loc
 static int defaultWidth[] =    {  70, 140, 90,  50,  50,  50,  50,  70,  50,  50,  70,  50,  50, 500};
@@ -824,6 +807,7 @@ void DiveListView::contextMenuEvent(QContextMenuEvent *event)
 		popup.addAction(tr("Shift times"), this, SLOT(shiftTimes()));
 		popup.addAction(tr("Load images"), this, SLOT(loadImages()));
 	}
+
 	// "collapse all" really closes all trips,
 	// "collapse" keeps the trip with the selected dive open
 	QAction *actionTaken = popup.exec(event->globalPos());
@@ -836,6 +820,7 @@ void DiveListView::contextMenuEvent(QContextMenuEvent *event)
 	event->accept();
 }
 
+
 void DiveListView::shiftTimes()
 {
 	ShiftTimesDialog::instance()->show();
