@@ -1,6 +1,9 @@
 #include "downloadfromdivecomputer.h"
 #include "helpers.h"
 #include "mainwindow.h"
+#include "divelistview.h"
+#include "display.h"
+#include "uemis.h"
 
 #include <QTimer>
 #include <QFileDialog>
@@ -282,6 +285,14 @@ void DownloadFromDCWidget::on_downloadCancelRetryButton_clicked()
 		diveImportedModel->clearTable();
 		clear_table(&downloadTable);
 	}
+	if (ui.vendor->currentText() == "Uemis") {
+		if (currentState == ERROR && downloadTable.nr > 0)
+			// let the uemis code know how far we've gotten
+			uemis_set_max_diveid_from_dialog(downloadTable.dives[downloadTable.nr - 1]->dc.diveid);
+		else
+			// fresh download, so only look at what's in the dive_table
+			uemis_set_max_diveid_from_dialog(0);
+	}
 	updateState(DOWNLOADING);
 
 	// you cannot cancel the dialog, just the download
@@ -294,10 +305,20 @@ void DownloadFromDCWidget::on_downloadCancelRetryButton_clicked()
 		thread->deleteLater();
 	}
 
-	data.devname = strdup(ui.device->currentText().toUtf8().data());
 	data.vendor = strdup(ui.vendor->currentText().toUtf8().data());
 	data.product = strdup(ui.product->currentText().toUtf8().data());
+	if (same_string(data.vendor, "Uemis")) {
+		char *colon;
+		char *devname = strdup(ui.device->currentText().toUtf8().data());
 
+		if ((colon = strstr(devname, ":\\ (UEMISSDA)")) != NULL) {
+			*(colon + 2) = '\0';
+			fprintf(stderr, "shortened devname to \"%s\"", data.devname);
+		}
+		data.devname = devname;
+	} else {
+		data.devname = strdup(ui.device->currentText().toUtf8().data());
+	}
 	data.descriptor = descriptorLookup[ui.vendor->currentText() + ui.product->currentText()];
 	data.force_download = ui.forceDownload->isChecked();
 	data.create_new_trip = ui.createNewTrip->isChecked();
@@ -420,7 +441,7 @@ void DownloadFromDCWidget::on_ok_clicked()
 {
 	struct dive *dive;
 
-	if (currentState != DONE)
+	if (currentState != DONE && currentState != ERROR)
 		return;
 
 	// record all the dives in the 'real' dive_table
