@@ -4,10 +4,13 @@
 
 #include <marble/MarbleWidget.h>
 #include <marble/GeoDataCoordinates.h>
-#include <marble/GeoDataDocument.h>
 
 #include <QHash>
 
+namespace Marble{
+	class GeoDataDocument;
+}
+
 class KMessageWidget;
 using namespace Marble;
 struct dive;
@@ -34,6 +37,10 @@ private:
 	int currentZoomLevel;
 	bool needResetZoom;
 	bool editingDiveLocation;
+	bool doubleClick;
+
+signals:
+	void coordinatesChanged(void);
 
 public
 slots:
@@ -54,9 +61,10 @@ public:
 	GlobeGPS(QWidget *parent);
 	void reload();
 	void repopulateLabels();
-	void centerOn(struct dive* dive);
-	bool eventFilter(QObject*, QEvent*);
-public slots:
+	void centerOnCurrentDive();
+	bool eventFilter(QObject *, QEvent *);
+public
+slots:
 	void prepareForGetDiveCoordinates();
 };
 
