@@ -71,6 +71,7 @@ slots:
 	void on_suit_textChanged(const QString &text);
 	void on_notes_textChanged();
 	void on_airtemp_textChanged(const QString &text);
+	void divetype_Changed(int);
 	void on_watertemp_textChanged(const QString &text);
 	void validate_temp_field(QLineEdit *tempField, const QString &text);
 	void on_dateEdit_dateChanged(const QDate &date);
@@ -91,6 +92,8 @@ slots:
 	void escDetected(void);
 	void photoDoubleClicked(const QString filePath);
 	void removeSelectedPhotos();
+	void updateGpsCoordinates();
+
 private:
 	Ui::MainTab ui;
 	WeightModel *weightModel;
@@ -108,10 +111,10 @@ private:
 	bool copyPaste;
 	void resetPallete();
 	void saveTags();
-	void updateGpsCoordinates(const struct dive *dive);
 	void markChangedWidget(QWidget *w);
 	dive_trip_t *currentTrip;
 	dive_trip_t displayedTrip;
+	bool acceptingEdit;
 };
 
 #endif // MAINTAB_H
