@@ -1,24 +1,9 @@
 #include "modeldelegates.h"
 #include "dive.h"
-#include "divelist.h"
-#include "starwidget.h"
-#include "models.h"
-#include "diveplanner.h"
-#include "simplewidgets.h"
 #include "gettextfromc.h"
+#include "mainwindow.h"
 
-#include <QtDebug>
-#include <QPainter>
-#include <QSortFilterProxyModel>
-#include <QStyle>
-#include <QStyleOption>
-#include <QComboBox>
 #include <QCompleter>
-#include <QLineEdit>
-#include <QKeyEvent>
-#include <QAbstractItemView>
-#include <QApplication>
-#include <QTextDocument>
 
 QSize DiveListDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
@@ -102,6 +87,7 @@ struct CurrSelected {
 
 QWidget *ComboBoxDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
+	MainWindow *m = MainWindow::instance();
 	QComboBox *comboDelegate = new QComboBox(parent);
 	comboDelegate->setModel(model);
 	comboDelegate->setEditable(true);
@@ -110,6 +96,8 @@ QWidget *ComboBoxDelegate::createEditor(QWidget *parent, const QStyleOptionViewI
 	comboDelegate->completer()->setCompletionMode(QCompleter::PopupCompletion);
 	comboDelegate->view()->setEditTriggers(QAbstractItemView::AllEditTriggers);
 	comboDelegate->lineEdit()->installEventFilter(const_cast<QObject *>(qobject_cast<const QObject *>(this)));
+	if ((m->graphics()->currentState != ProfileWidget2::PROFILE))
+		comboDelegate->lineEdit()->setEnabled(false);
 	comboDelegate->view()->installEventFilter(const_cast<QObject *>(qobject_cast<const QObject *>(this)));
 	QAbstractItemView *comboPopup = comboDelegate->lineEdit()->completer()->popup();
 	comboPopup->setMouseTracking(true);
@@ -252,6 +240,16 @@ void TankInfoDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
 
 TankInfoDelegate::TankInfoDelegate(QObject *parent) : ComboBoxDelegate(TankInfoModel::instance(), parent)
 {
+	connect(this, SIGNAL(closeEditor(QWidget *, QAbstractItemDelegate::EndEditHint)),
+		this, SLOT(reenableReplot(QWidget *, QAbstractItemDelegate::EndEditHint)));
+}
+
+void TankInfoDelegate::reenableReplot(QWidget *widget, QAbstractItemDelegate::EndEditHint hint)
+{
+	MainWindow::instance()->graphics()->setReplot(true);
+	// FIXME: We need to replot after a cylidner is selected but the replot below overwrites
+	//        the newly selected cylinder.
+	//	MainWindow::instance()->graphics()->replot();
 }
 
 void TankInfoDelegate::revertModelData(QWidget *widget, QAbstractItemDelegate::EndEditHint hint)
@@ -275,12 +273,12 @@ QWidget *TankInfoDelegate::createEditor(QWidget *parent, const QStyleOptionViewI
 	currCylinderData.type = copy_string(cyl->type.description);
 	currCylinderData.pressure = cyl->type.workingpressure.mbar;
 	currCylinderData.size = cyl->type.size.mliter;
+	MainWindow::instance()->graphics()->setReplot(false);
 	return delegate;
 }
 
-TankUseDelegate::TankUseDelegate(QObject *parent)
+TankUseDelegate::TankUseDelegate(QObject *parent) : QStyledItemDelegate(parent)
 {
-
 }
 
 QWidget *TankUseDelegate::createEditor(QWidget * parent, const QStyleOptionViewItem & option, const QModelIndex & index) const
