@@ -1,46 +1,45 @@
 #include "divetextitem.h"
-#include "animationfunctions.h"
 #include "mainwindow.h"
 
-#include <QPropertyAnimation>
-#include <QApplication>
-#include <QFont>
-#include <QFontMetrics>
-#include <QBrush>
-#include <QPen>
-#include <QDebug>
-
 DiveTextItem::DiveTextItem(QGraphicsItem *parent) : QGraphicsItemGroup(parent),
 	internalAlignFlags(Qt::AlignHCenter | Qt::AlignVCenter),
-	textBackgroundItem(NULL),
-	textItem(NULL),
-	colorIndex(SAC_DEFAULT),
+	textBackgroundItem(new QGraphicsPathItem(this)),
+	textItem(new QGraphicsPathItem(this)),
 	scale(1.0)
 {
 	setFlag(ItemIgnoresTransformations);
+	textBackgroundItem->setBrush(QBrush(getColor(TEXT_BACKGROUND)));
+	textBackgroundItem->setPen(Qt::NoPen);
+	textItem->setPen(Qt::NoPen);
 }
 
 void DiveTextItem::setAlignment(int alignFlags)
 {
-	internalAlignFlags = alignFlags;
-	updateText();
+	if (alignFlags != internalAlignFlags) {
+		internalAlignFlags = alignFlags;
+		updateText();
+	}
 }
 
 void DiveTextItem::setBrush(const QBrush &b)
 {
-	brush = b;
-	updateText();
+	textItem->setBrush(b);
 }
 
 void DiveTextItem::setScale(double newscale)
 {
-	scale = newscale;
+	if (scale != newscale) {
+		scale = newscale;
+		updateText();
+	}
 }
 
 void DiveTextItem::setText(const QString &t)
 {
-	internalText = t;
-	updateText();
+	if (internalText != t) {
+		internalText = t;
+		updateText();
+	}
 }
 
 const QString &DiveTextItem::text()
@@ -51,10 +50,6 @@ const QString &DiveTextItem::text()
 void DiveTextItem::updateText()
 {
 	double size;
-	delete textItem;
-	textItem = NULL;
-	delete textBackgroundItem;
-	textBackgroundItem = NULL;
 	if (internalText.isEmpty()) {
 		return;
 	}
@@ -76,21 +71,16 @@ void DiveTextItem::updateText()
 
 	QRectF rect = fm.boundingRect(internalText);
 	yPos = (internalAlignFlags & Qt::AlignTop) ? 0 :
-						     (internalAlignFlags & Qt::AlignBottom) ? +rect.height() :
-											      /*(internalAlignFlags & Qt::AlignVCenter  ? */ +rect.height() / 4;
+		(internalAlignFlags & Qt::AlignBottom) ? +rect.height() :
+		/*(internalAlignFlags & Qt::AlignVCenter  ? */ +rect.height() / 4;
 
 	xPos = (internalAlignFlags & Qt::AlignLeft) ? -rect.width() :
-						      (internalAlignFlags & Qt::AlignHCenter) ? -rect.width() / 2 :
-												/* (internalAlignFlags & Qt::AlignRight) */ 0;
+		(internalAlignFlags & Qt::AlignHCenter) ? -rect.width() / 2 :
+		/* (internalAlignFlags & Qt::AlignRight) */ 0;
 
 	textPath.addText(xPos, yPos, fnt, internalText);
 	QPainterPathStroker stroker;
 	stroker.setWidth(3);
-	textBackgroundItem = new QGraphicsPathItem(stroker.createStroke(textPath), this);
-	textBackgroundItem->setBrush(QBrush(getColor(TEXT_BACKGROUND)));
-	textBackgroundItem->setPen(Qt::NoPen);
-
-	textItem = new QGraphicsPathItem(textPath, this);
-	textItem->setBrush(brush);
-	textItem->setPen(Qt::NoPen);
+	textBackgroundItem->setPath(stroker.createStroke(textPath));
+	textItem->setPath(textPath);
 }
