@@ -1,49 +1,44 @@
 #include "divetooltipitem.h"
 #include "divecartesianaxis.h"
-#include "profilewidget2.h"
 #include "dive.h"
 #include "profile.h"
 #include "membuffer.h"
 #include "metrics.h"
 #include <QPropertyAnimation>
-#include <QGraphicsSceneMouseEvent>
-#include <QPen>
-#include <QBrush>
-#include <QGraphicsScene>
 #include <QSettings>
 #include <QGraphicsView>
-#include <QDebug>
+#include <QStyleOptionGraphicsItem>
 
 #define PORT_IN_PROGRESS 1
 #ifdef PORT_IN_PROGRESS
 #include "display.h"
 #endif
 
-void ToolTipItem::addToolTip(const QString &toolTip, const QIcon &icon, const QPixmap *pixmap)
+void ToolTipItem::addToolTip(const QString &toolTip, const QIcon &icon, const QPixmap& pixmap)
 {
 	const IconMetrics& iconMetrics = defaultIconMetrics();
 
-	QGraphicsPixmapItem *iconItem = 0, *pixmapItem = 0;
+	QGraphicsPixmapItem *iconItem = 0;
 	double yValue = title->boundingRect().height() + iconMetrics.spacing;
 	Q_FOREACH (ToolTip t, toolTips) {
 		yValue += t.second->boundingRect().height();
 	}
-	if (!icon.isNull()) {
-		iconItem = new QGraphicsPixmapItem(icon.pixmap(iconMetrics.sz_small, iconMetrics.sz_small), this);
-		iconItem->setPos(iconMetrics.spacing, yValue);
-	} else {
-		if (pixmap && !pixmap->isNull()) {
-			pixmapItem = new QGraphicsPixmapItem(*pixmap, this);
-			pixmapItem->setPos(iconMetrics.spacing, yValue);
-		}
+	if (entryToolTip.second) {
+		yValue += entryToolTip.second->boundingRect().height();
 	}
+	iconItem = new QGraphicsPixmapItem(this);
+	if (!icon.isNull()) {
+		iconItem->setPixmap(icon.pixmap(iconMetrics.sz_small, iconMetrics.sz_small));
+	} else if (!pixmap.isNull()) {
+		iconItem->setPixmap(pixmap);
+	}
+	iconItem->setPos(iconMetrics.spacing, yValue);
 
 	QGraphicsSimpleTextItem *textItem = new QGraphicsSimpleTextItem(toolTip, this);
 	textItem->setPos(iconMetrics.spacing + iconMetrics.sz_small + iconMetrics.spacing, yValue);
 	textItem->setBrush(QBrush(Qt::white));
 	textItem->setFlag(ItemIgnoresTransformations);
 	toolTips.push_back(qMakePair(iconItem, textItem));
-	expand();
 }
 
 void ToolTipItem::clear()
@@ -57,33 +52,11 @@ void ToolTipItem::clear()
 
 void ToolTipItem::setRect(const QRectF &r)
 {
-	// qDeleteAll(childItems());
-	delete background;
-
-	rectangle = r;
-	setBrush(QBrush(Qt::white));
-	setPen(QPen(Qt::black, 0.5));
-
-	// Creates a 2pixels border
-	QPainterPath border;
-	border.addRoundedRect(-4, -4, rectangle.width() + 8, rectangle.height() + 10, 3, 3);
-	border.addRoundedRect(-1, -1, rectangle.width() + 3, rectangle.height() + 4, 3, 3);
-	setPath(border);
-
-	QPainterPath bg;
-	bg.addRoundedRect(-1, -1, rectangle.width() + 3, rectangle.height() + 4, 3, 3);
-
-	QColor c = QColor(Qt::black);
-	c.setAlpha(155);
-
-	QGraphicsPathItem *b = new QGraphicsPathItem(bg, this);
-	b->setFlag(ItemStacksBehindParent);
-	b->setFlag(ItemIgnoresTransformations);
-	b->setBrush(c);
-	b->setPen(QPen(QBrush(Qt::transparent), 0));
-	b->setZValue(-10);
-	background = b;
+	if( r == rect() ) {
+		return;
+	}
 
+	QGraphicsRectItem::setRect(r);
 	updateTitlePosition();
 }
 
@@ -91,11 +64,15 @@ void ToolTipItem::collapse()
 {
 	int dim = defaultIconMetrics().sz_small;
 
-	QPropertyAnimation *animation = new QPropertyAnimation(this, "rect");
-	animation->setDuration(100);
-	animation->setStartValue(nextRectangle);
-	animation->setEndValue(QRect(0, 0, dim, dim));
-	animation->start(QAbstractAnimation::DeleteWhenStopped);
+	if (prefs.animation_speed) {
+		QPropertyAnimation *animation = new QPropertyAnimation(this, "rect");
+		animation->setDuration(100);
+		animation->setStartValue(nextRectangle);
+		animation->setEndValue(QRect(0, 0, dim, dim));
+		animation->start(QAbstractAnimation::DeleteWhenStopped);
+	} else {
+		setRect(nextRectangle);
+	}
 	clear();
 
 	status = COLLAPSED;
@@ -109,11 +86,20 @@ void ToolTipItem::expand()
 	const IconMetrics& iconMetrics = defaultIconMetrics();
 
 	double width = 0, height = title->boundingRect().height() + iconMetrics.spacing;
-	Q_FOREACH (ToolTip t, toolTips) {
-		if (t.second->boundingRect().width() > width)
-			width = t.second->boundingRect().width();
-		height += t.second->boundingRect().height();
+	Q_FOREACH (const ToolTip& t, toolTips) {
+		QRectF sRect = t.second->boundingRect();
+		if (sRect.width() > width)
+			width = sRect.width();
+		height += sRect.height();
 	}
+
+	if (entryToolTip.first) {
+		QRectF sRect = entryToolTip.second->boundingRect();
+		if (sRect.width() > width)
+			width = sRect.width();
+		height += sRect.height();
+	}
+
 	/*       Left padding, Icon Size,   space, right padding */
 	width += iconMetrics.spacing + iconMetrics.sz_small + iconMetrics.spacing + iconMetrics.spacing;
 
@@ -126,28 +112,48 @@ void ToolTipItem::expand()
 	nextRectangle.setWidth(width);
 	nextRectangle.setHeight(height);
 
-	QPropertyAnimation *animation = new QPropertyAnimation(this, "rect");
-	animation->setDuration(100);
-	animation->setStartValue(rectangle);
-	animation->setEndValue(nextRectangle);
-	animation->start(QAbstractAnimation::DeleteWhenStopped);
+	if (nextRectangle != rect()) {
+		if (prefs.animation_speed) {
+			QPropertyAnimation *animation = new QPropertyAnimation(this, "rect", this);
+			animation->setDuration(prefs.animation_speed);
+			animation->setStartValue(rect());
+			animation->setEndValue(nextRectangle);
+			animation->start(QAbstractAnimation::DeleteWhenStopped);
+		} else {
+			setRect(nextRectangle);
+		}
+	}
 
 	status = EXPANDED;
 }
 
-ToolTipItem::ToolTipItem(QGraphicsItem *parent) : QGraphicsPathItem(parent),
-	background(0),
-	separator(new QGraphicsLineItem(this)),
+ToolTipItem::ToolTipItem(QGraphicsItem *parent) : QGraphicsRectItem(parent),
 	title(new QGraphicsSimpleTextItem(tr("Information"), this)),
 	status(COLLAPSED),
 	timeAxis(0),
 	lastTime(-1)
 {
 	memset(&pInfo, 0, sizeof(pInfo));
-
+	entryToolTip.first = NULL;
+	entryToolTip.second = NULL;
 	setFlags(ItemIgnoresTransformations | ItemIsMovable | ItemClipsChildrenToShape);
-	updateTitlePosition();
+
+	QColor c = QColor(Qt::black);
+	c.setAlpha(155);
+	setBrush(c);
+
 	setZValue(99);
+
+	addToolTip(QString(), QIcon(), QPixmap(16,60));
+	entryToolTip = toolTips.first();
+	toolTips.clear();
+
+	title->setFlag(ItemIgnoresTransformations);
+	title->setPen(QPen(Qt::white, 1));
+	title->setBrush(Qt::white);
+
+	setPen(QPen(Qt::white, 2));
+	refreshTime.start();
 }
 
 ToolTipItem::~ToolTipItem()
@@ -158,31 +164,14 @@ ToolTipItem::~ToolTipItem()
 void ToolTipItem::updateTitlePosition()
 {
 	const IconMetrics& iconMetrics = defaultIconMetrics();
-	if (rectangle.width() < title->boundingRect().width() + iconMetrics.spacing * 4) {
-		QRectF newRect = rectangle;
+	if (rect().width() < title->boundingRect().width() + iconMetrics.spacing * 4) {
+		QRectF newRect = rect();
 		newRect.setWidth(title->boundingRect().width() + iconMetrics.spacing * 4);
 		newRect.setHeight((newRect.height() && isExpanded()) ? newRect.height() : iconMetrics.sz_small);
 		setRect(newRect);
 	}
 
-	title->setPos(boundingRect().width() / 2 - title->boundingRect().width() / 2 - 1, 0);
-	title->setFlag(ItemIgnoresTransformations);
-	title->setPen(QPen(Qt::white, 1));
-	title->setBrush(Qt::white);
-
-	if (toolTips.size() > 0) {
-		double x1 = 3;
-		double y1 = title->pos().y() + iconMetrics.spacing / 2 + title->boundingRect().height();
-		double x2 = boundingRect().width() - 10;
-		double y2 = y1;
-
-		separator->setLine(x1, y1, x2, y2);
-		separator->setFlag(ItemIgnoresTransformations);
-		separator->setPen(QPen(Qt::white));
-		separator->show();
-	} else {
-		separator->hide();
-	}
+	title->setPos(rect().width() / 2 - title->boundingRect().width() / 2 - 1, 0);
 }
 
 bool ToolTipItem::isExpanded() const
@@ -193,12 +182,23 @@ bool ToolTipItem::isExpanded() const
 void ToolTipItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 {
 	persistPos();
-	QGraphicsPathItem::mouseReleaseEvent(event);
+	QGraphicsRectItem::mouseReleaseEvent(event);
 	Q_FOREACH (QGraphicsItem *item, oldSelection) {
 		item->setSelected(true);
 	}
 }
 
+void ToolTipItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
+{
+	Q_UNUSED(widget);
+	painter->save();
+	painter->setClipRect(option->rect);
+	painter->setPen(pen());
+	painter->setBrush(brush());
+	painter->drawRoundedRect(rect(), 10, 10, Qt::AbsoluteSize);
+	painter->restore();
+}
+
 void ToolTipItem::persistPos()
 {
 	QSettings s;
@@ -230,45 +230,51 @@ void ToolTipItem::setTimeAxis(DiveCartesianAxis *axis)
 
 void ToolTipItem::refresh(const QPointF &pos)
 {
-	int i;
 	struct plot_data *entry;
-	static QPixmap *tissues = new QPixmap(16,60);
-	static QPainter *painter = new QPainter(tissues);
+	static QPixmap tissues(16,60);
+	static QPainter painter(&tissues);
+	static struct membuffer mb = { 0 };
+
+	if(refreshTime.elapsed() < 40)
+		return;
+	refreshTime.start();
+
 	int time = timeAxis->valueAt(pos);
 	if (time == lastTime)
 		return;
 
 	lastTime = time;
 	clear();
-	struct membuffer mb = { 0 };
 
+	mb.len = 0;
 	entry = get_plot_details_new(&pInfo, time, &mb);
 	if (entry) {
-		tissues->fill();
-		painter->setPen(QColor(0, 0, 0, 0));
-		painter->setBrush(QColor(LIMENADE1));
-		painter->drawRect(0, 10 + (100 - AMB_PERCENTAGE) / 2, 16, AMB_PERCENTAGE / 2);
-		painter->setBrush(QColor(SPRINGWOOD1));
-		painter->drawRect(0, 10, 16, (100 - AMB_PERCENTAGE) / 2);
-		painter->setBrush(QColor("Red"));
-		painter->drawRect(0,0,16,10);
-		painter->setPen(QColor(0, 0, 0, 255));
-		painter->drawLine(0, 60 - entry->gfline / 2, 16, 60 - entry->gfline / 2);
-		painter->drawLine(0, 60 - AMB_PERCENTAGE * (entry->pressures.n2 + entry->pressures.he) / entry->ambpressure / 2,
+		tissues.fill();
+		painter.setPen(QColor(0, 0, 0, 0));
+		painter.setBrush(QColor(LIMENADE1));
+		painter.drawRect(0, 10 + (100 - AMB_PERCENTAGE) / 2, 16, AMB_PERCENTAGE / 2);
+		painter.setBrush(QColor(SPRINGWOOD1));
+		painter.drawRect(0, 10, 16, (100 - AMB_PERCENTAGE) / 2);
+		painter.setBrush(QColor(Qt::red));
+		painter.drawRect(0,0,16,10);
+		painter.setPen(QColor(0, 0, 0, 255));
+		painter.drawLine(0, 60 - entry->gfline / 2, 16, 60 - entry->gfline / 2);
+		painter.drawLine(0, 60 - AMB_PERCENTAGE * (entry->pressures.n2 + entry->pressures.he) / entry->ambpressure / 2,
 				16, 60 - AMB_PERCENTAGE * (entry->pressures.n2 + entry->pressures.he) / entry->ambpressure /2);
-		painter->setPen(QColor(0, 0, 0, 127));
-		for (i=0; i<16; i++) {
-			painter->drawLine(i, 60, i, 60 - entry->percentages[i] / 2);
+		painter.setPen(QColor(0, 0, 0, 127));
+		for (int i=0; i<16; i++) {
+			painter.drawLine(i, 60, i, 60 - entry->percentages[i] / 2);
 		}
-		addToolTip(QString::fromUtf8(mb.buffer, mb.len),QIcon(), tissues);
+		entryToolTip.first->setPixmap(tissues);
+		entryToolTip.second->setText(QString::fromUtf8(mb.buffer, mb.len));
 	}
-	free_buffer(&mb);
 
-	Q_FOREACH (QGraphicsItem *item, scene()->items(pos, Qt::IntersectsItemShape
+	Q_FOREACH (QGraphicsItem *item, scene()->items(pos, Qt::IntersectsItemBoundingRect
 		,Qt::DescendingOrder, scene()->views().first()->transform())) {
 		if (!item->toolTip().isEmpty())
 			addToolTip(item->toolTip());
 	}
+	expand();
 }
 
 void ToolTipItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
