@@ -3,6 +3,7 @@
 
 class MinMaxAvgWidgetPrivate;
 class QAbstractButton;
+class QNetworkReply;
 
 #include <QWidget>
 #include <QDialog>
@@ -14,9 +15,11 @@ class QAbstractButton;
 #include "ui_shiftimagetimes.h"
 #include "ui_divecomponentselection.h"
 #include "ui_listfilter.h"
+#include "ui_filterwidget.h"
 #include "exif.h"
 #include <dive.h>
 
+
 class MinMaxAvgWidget : public QWidget {
 	Q_OBJECT
 	Q_PROPERTY(double minimum READ minimum WRITE setMinimum)
@@ -163,7 +166,7 @@ slots:
 
 public:
 	MultiFilter(QWidget *parent);
-	Ui::FilterWidget2 *ui;
+	Ui::FilterWidget2 ui;
 };
 
 class TagFilter : public QWidget {
