@@ -29,6 +29,8 @@ struct preferences default_prefs = {
 	.gfhigh = 75,
 	.animation_speed = 500,
 	.gf_low_at_maxdepth = false,
+	.show_ccr_setpoint = false,
+	.show_ccr_sensors = false,
 	.font_size = -1,
 	.display_invalid_dives = false,
 	.show_sac = false,
@@ -45,10 +47,16 @@ struct preferences default_prefs = {
 	.drop_stone_mode = false,
 	.bottomsac = 20000,
 	.decosac = 17000,
-	.o2consumption = 1000,
+	.o2consumption = 720,
 	.pscr_ratio = 100,
 	.show_pictures_in_profile = true,
-	.tankbar = false
+	.tankbar = false,
+	.facebook = {
+		.user_id = NULL,
+		.album_id = NULL,
+		.access_token = NULL
+	},
+	.defaultsetpoint = 1100
 };
 
 int run_survey;
@@ -102,7 +110,7 @@ bool imported = false;
 
 static void print_version()
 {
-	printf("Subsurface v%s, ", VERSION_STRING);
+	printf("Subsurface v%s, ", GIT_VERSION_STRING);
 	printf("built with libdivecomputer v%s\n", dc_version(NULL));
 }
 
@@ -219,3 +227,20 @@ void setup_system_prefs(void)
 
 	default_prefs.units = IMPERIAL_units;
 }
+
+/*
+ * Free strduped prefs before exit.
+ *
+ * These are not real leaks but they plug the holes found by eg.
+ * valgrind so you can find the real leaks.
+ */
+void free_prefs(void)
+{
+	free((void*)prefs.default_filename);
+	free((void*)prefs.default_cylinder);
+	free((void*)prefs.divelist_font);
+	free(prefs.proxy_host);
+	free(prefs.proxy_user);
+	free(prefs.proxy_pass);
+	free(prefs.userid);
+}
