@@ -15,6 +15,7 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
+#include <errno.h>
 
 #include "gettext.h"
 #include "libdivecomputer.h"
@@ -440,6 +441,11 @@ static bool uemis_get_answer(const char *path, char *request, int n_param_in,
 	int timeout = UEMIS_LONG_TIMEOUT;
 
 	reqtxt_file = subsurface_open(reqtxt_path, O_RDWR | O_CREAT, 0666);
+	if (reqtxt_file == -1) {
+		*error_text = "can't open req.txt";
+		fprintf(stderr, "open %s failed with errno %d\n", reqtxt_path, errno);
+		return false;
+	}
 	snprintf(sb, BUFLEN, "n%04d12345678", filenr);
 	str_append_with_delim(sb, request);
 	for (i = 0; i < n_param_in; i++)
@@ -635,7 +641,7 @@ static void parse_divespot(char *buf)
 				latitude = ascii_strtod(val, NULL);
 		}
 	} while (tag && *tag);
-	uemis_set_divelocation(divespot, locationstring, latitude, longitude);
+	uemis_set_divelocation(divespot, locationstring, longitude, latitude);
 }
 
 static void track_divespot(char *val, int diveid, char **location, degrees_t *latitude, degrees_t *longitude)
@@ -814,6 +820,13 @@ static bool process_raw_buffer(device_data_t *devdata, uint32_t deviceid, char *
 	return true;
 }
 
+static int max_diveid_from_dialog;
+
+void uemis_set_max_diveid_from_dialog(int diveid)
+{
+	max_diveid_from_dialog = diveid;
+}
+
 static char *uemis_get_divenr(char *deviceidstr)
 {
 	uint32_t deviceid, maxdiveid = 0;
@@ -831,7 +844,7 @@ static char *uemis_get_divenr(char *deviceidstr)
 				maxdiveid = dc->diveid;
 		}
 	}
-	snprintf(divenr, 10, "%d", maxdiveid);
+	snprintf(divenr, 10, "%d", maxdiveid > max_diveid_from_dialog ? maxdiveid : max_diveid_from_dialog);
 	return strdup(divenr);
 }
 
@@ -840,7 +853,8 @@ const char *do_uemis_import(device_data_t *data)
 	const char *mountpath = data->devname;
 	short force_download = data->force_download;
 	char *newmax = NULL;
-	int start, end = -2, i, offset;
+	int first, start, end = -2;
+	int i, offset = 0;
 	uint32_t deviceidnr;
 	char objectid[10];
 	char *deviceid = NULL;
@@ -870,13 +884,13 @@ const char *do_uemis_import(device_data_t *data)
 	if (import_thread_cancelled)
 		goto bail;
 	param_buff[1] = "notempty";
-	/* if we have an empty divelist or force it, then we start downloading from the
-	 * first dive on the Uemis; otherwise check which was the last dive downloaded */
-	if (!force_download && dive_table.nr > 0)
+	/* if we force it we start downloading from the first dive on the Uemis;
+	 *  otherwise check which was the last dive downloaded */
+	if (!force_download)
 		newmax = uemis_get_divenr(deviceid);
 	else
 		newmax = strdup("0");
-	start = atoi(newmax);
+	first = start = atoi(newmax);
 	for (;;) {
 #if UEMIS_DEBUG & 4
 		fprintf(debugfile, "d_u_i inner loop start %d end %d newmax %s\n", start, end, newmax);
@@ -899,9 +913,55 @@ const char *do_uemis_import(device_data_t *data)
 			free(t);
 			once = false;
 		}
+		/* clean up mbuf */
+		endptr = strstr(mbuf, "{{{");
+		if (endptr)
+			*(endptr + 2) = '\0';
+		/* last object_id we parsed */
+		sscanf(newmax, "%d", &end);
 #if UEMIS_DEBUG & 4
-		fprintf(debugfile, "d_u_i after download and parse start %d end %d newmax %s\n", start, end, newmax);
+		fprintf(debugfile, "d_u_i after download and parse start %d end %d newmax %s progress %4.2f\n", start, end, newmax, progress_bar_fraction);
 #endif
+		/* now download the additional dive data with "getDive" for the dives
+		 * we just downloaded - yes, this is completely insane - why isn't all of
+		 * this downloaded in the first place??? */
+		for (i = start; i <= end; i++) {
+			snprintf(objectid, sizeof(objectid), "%d", i + offset);
+			param_buff[2] = objectid;
+#if UEMIS_DEBUG & 2
+			fprintf(debugfile, "getDive %d, object_id %s\n", i, objectid);
+#endif
+			/* there is no way I have found to directly get the dive information
+			 * for dive #i as the object_id and logfilenr can be different in the
+			 * getDive call; so we get the first one, compare the actual divenr
+			 * with the one that we wanted, calculate the offset and try again.
+			 * What an insane design... */
+			success = uemis_get_answer(mountpath, "getDive", 3, 0, &result);
+			if (mbuf) {
+				int divenr;
+				(void)process_raw_buffer(data, deviceidnr, mbuf, &newmax, false, &divenr);
+#if UEMIS_DEBUG & 2
+				fprintf(debugfile, "got dive %d, looking for dive %d\n", divenr, i);
+#endif
+				if (divenr != i) {
+					if (divenr == -1) {
+						offset--;
+					} else {
+						offset += i - divenr;
+					}
+#if UEMIS_DEBUG & 2
+					fprintf(debugfile, " -> trying again with offset %d\n", offset);
+#endif
+					i = start - 1;
+					if (i + offset < 0)
+						break;
+					continue;
+				}
+			}
+			if (!success || import_thread_cancelled)
+				break;
+		}
+		start = end + 1;
 		/* if the user clicked cancel, exit gracefully */
 		if (import_thread_cancelled)
 			goto bail;
@@ -909,60 +969,17 @@ const char *do_uemis_import(device_data_t *data)
 		if (!success || !param_buff[3])
 			break;
 		/* finally, if the memory is getting too full, maybe we better stop, too */
-		if (progress_bar_fraction > 0.85) {
+		if (progress_bar_fraction > 0.80) {
 			result = translate("gettextFromC", ERR_FS_ALMOST_FULL);
 			break;
 		}
-		/* clean up mbuf */
-		endptr = strstr(mbuf, "{{{");
-		if (endptr)
-			*(endptr + 2) = '\0';
-		/* last object_id we parsed */
-		sscanf(newmax, "%d", &end);
 	}
 	if (end == -2 && sscanf(newmax, "%d", &end) != 1)
 		end = start;
 #if UEMIS_DEBUG & 2
-	fprintf(debugfile, "done: read from object_id %d to %d\n", start, end);
+	fprintf(debugfile, "done: read from object_id %d to %d\n", first, end);
 #endif
 	free(newmax);
-	offset = 0;
-	for (i = start; i <= end; i++) {
-		snprintf(objectid, sizeof(objectid), "%d", i + offset);
-		param_buff[2] = objectid;
-#if UEMIS_DEBUG & 2
-		fprintf(debugfile, "getDive %d, object_id %s\n", i, objectid);
-#endif
-		/* there is no way I have found to directly get the dive information
-		 * for dive #i as the object_id and logfilenr can be different in the
-		 * getDive call; so we get the first one, compare the actual divenr
-		 * with the one that we wanted, calculate the offset and try again.
-		 * What an insane design... */
-		success = uemis_get_answer(mountpath, "getDive", 3, 0, &result);
-		if (mbuf) {
-			int divenr;
-			(void)process_raw_buffer(data, deviceidnr, mbuf, &newmax, false, &divenr);
-#if UEMIS_DEBUG & 2
-			fprintf(debugfile, "got dive %d, looking for dive %d\n", divenr, i);
-#endif
-			if (divenr != i) {
-				if (divenr == -1) {
-					offset--;
-				} else {
-					offset += i - divenr;
-				}
-#if UEMIS_DEBUG & 2
-				fprintf(debugfile, " -> trying again with offset %d\n", offset);
-#endif
-				i = start - 1;
-				if (i + offset < 0)
-					break;
-				continue;
-			}
-		}
-		if (!success || import_thread_cancelled)
-			break;
-	}
 	success = true;
 	for (i = 0; i <= nr_divespots; i++) {
 		char divespotnr[10];
